// Copyright (c) Mito
import React, { useRef, useState } from 'react';
import "../../../../css/taskpanes/ControlPanel/ControlPanelTaskpane.css";
import { useDebouncedEffect } from '../../../hooks/useDebouncedEffect';
import { getCellDataFromCellIndexes } from '../../endo/utils';
import { TaskpaneType } from '../taskpanes';
import ColumnNameCard from './ColumnNameCard';
import ControlPanelTaskpaneTabs from './ControlPanelTaskpaneTabs';
import DtypeCard from './FilterAndSortTab/DtypeCard';
import FilterCard from './FilterAndSortTab/filter/FilterCard';
import { isFilterGroup } from './FilterAndSortTab/filter/filterTypes';
import { areFiltersEqual, getEmptyFilterData, getExclusiveFilterData, isValidFilter, parseFilter } from './FilterAndSortTab/filter/utils';
import SortCard from './FilterAndSortTab/SortCard';
import ColumnSummaryGraph from './SummaryStatsTab/ColumnSummaryGraph';
import ColumnSummaryStatistics from './SummaryStatsTab/ColumnSummaryStatistics';
import { ValuesTab } from './ValuesTab/ValuesTab';
/*
    We wait 500ms before sending a filter message to make sure
    that as the user is typing key changes, we don't queue up a
    ton of filtering messages.
*/
const FILTER_MESSAGE_DELAY = 500;
export var ControlPanelTab;
(function (ControlPanelTab) {
    ControlPanelTab["FilterSort"] = "filter_sort";
    ControlPanelTab["UniqueValues"] = "unique_values";
    ControlPanelTab["SummaryStats"] = "summary_stats";
})(ControlPanelTab || (ControlPanelTab = {}));
export const ControlPanelTaskpane = (props) => {
    // Get the values for the last cell that was selected
    const { columnHeader, columnID, columnFormula, columnMitoType, columnFilters } = getCellDataFromCellIndexes(props.sheetData, props.selection.endingRowIndex, props.selection.endingColumnIndex);
    const [filters, setFilters] = useState(columnFilters !== undefined ? columnFilters.filters : []);
    const [operator, setOperator] = useState(columnFilters !== undefined ? columnFilters.operator : 'And');
    const [stepID, setStepID] = useState('');
    // If this is not a valid column, don't render anything, and close the takspane! 
    // We have to do this after the useState calls, to make sure this is valid react
    if (columnHeader === undefined || columnID === undefined || columnFormula === undefined || columnMitoType === undefined) {
        props.setUIState((prevUIState) => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenTaskpane: { type: TaskpaneType.NONE } });
        });
        return React.createElement(React.Fragment, null);
    }
    // When the filters or operator changes, send a new message, as long as this is not
    // the first time that this rendered. We use a ref to avoid sending a message the first 
    // time it renders
    const firstRender = useRef(true);
    useDebouncedEffect(() => {
        if (!firstRender.current) {
            void _sendFilterUpdateMessage();
        }
        firstRender.current = false;
    }, [filters, operator], FILTER_MESSAGE_DELAY);
    /*
        NOTE: only call this through the sendFilterUpdateMessage function, to make sure
        buffering messages works properly.

        Before sending the displayed filters, we parse all the number filters from strings
        to numbers, and then filter out all of the invalid filters (as to not cause errors)
    */
    const _sendFilterUpdateMessage = async () => {
        // To handle decimals, we allow decimals to be submitted, and then just
        // parse them before they are sent to the back-end
        const parsedFilters = filters.map((filterOrGroup) => {
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.map((filter) => {
                        return parseFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return parseFilter(filterOrGroup);
            }
        });
        const filtersToApply = parsedFilters.map((filterOrGroup) => {
            // Filter out these incomplete filters from the group
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.filter((filter) => {
                        return isValidFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return filterOrGroup;
            }
        }).filter((filterOrGroup) => {
            // Filter out the groups if they have no valid filters in them
            if (isFilterGroup(filterOrGroup)) {
                return filterOrGroup.filters.length > 0;
            }
            // And then we filter the non group filters to be non-empty
            return isValidFilter(filterOrGroup);
        });
        const _stepID = await props.mitoAPI.sendFilterMessage(props.selectedSheetIndex, columnID, filtersToApply, operator, stepID, props.tab);
        setStepID(_stepID);
    };
    /*
        Adds a new, blank filter to the end of the filters list
    */
    const addFilter = () => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            newFilters.push(getEmptyFilterData(columnMitoType));
            return newFilters;
        });
    };
    /*
        Creates a new filter group (at the bottom) with a single empty
        filter
    */
    const addFilterGroup = () => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            newFilters.push({
                filters: [
                    getEmptyFilterData(columnMitoType)
                ],
                operator: 'And'
            });
            return newFilters;
        });
    };
    /*
        Adds a blank new filter to the end of a specific group
    */
    const addFilterToGroup = (groupIndex) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we add a new filter to it
                filterGroup.filters.push(getEmptyFilterData(columnMitoType));
                return newFilters;
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return prevFilters;
            }
        });
    };
    /*
        Toggles the exclusive filter for a specific value. An exclusive filter is the NOT_EXACTLY filter condition
        for strings, numbers, and dates. For booleans its either the IS_TRUE or IS_FALSE conditions.

        For a specific value in this column, the toggleExclusiveFilter determines if there is an exclusive filter that is excluding the value.
        If there is, it removes it. If there's is not, it applies it.
    */
    const toggleExclusiveFilters = (values) => {
        // Generate the filter
        setFilters((prevFilters) => {
            let newFilters = [...prevFilters];
            values.forEach(value => {
                const exclusiveFilter = getExclusiveFilterData(columnMitoType, value);
                const originalFilterLength = newFilters.length;
                // Remove the filter if it exists
                newFilters = newFilters.filter(filter => {
                    return isFilterGroup(filter) || !areFiltersEqual(filter, exclusiveFilter);
                });
                // If the filter didn't exist, then add it. 
                if (newFilters.length === originalFilterLength) {
                    newFilters.push(exclusiveFilter);
                }
            });
            return newFilters;
        });
    };
    /*
        Deletes a filter that is at the given index in the main
        filter list.
    */
    const deleteFilter = (filterIndex) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            newFilters.splice(filterIndex, 1);
            return newFilters;
        });
    };
    /*
        Deletes a filter that is at a given index in filter list
        of a specific filter group
    */
    const deleteFilterFromGroup = (groupIndex, filterIndex) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we delete the filter
                // at the passed filterIndex
                filterGroup.filters.splice(filterIndex, 1);
                // If there are no filters left in this group, then we remove the entire group
                if (filterGroup.filters.length === 0) {
                    newFilters.splice(groupIndex, 1);
                }
                return newFilters;
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return prevFilters;
            }
        });
    };
    /*
        Sets a filter at the given index to the new filter value
    */
    const setFilter = (filterIndex, filter) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            newFilters[filterIndex] = filter;
            return newFilters;
        });
    };
    /*
        Sets a filter at the given filterIndex in the specific group at the given
        groupIndex to the new filter value
    */
    const setFilterInGroup = (groupIndex, filterIndex, filter) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                filterGroup.filters[filterIndex] = filter;
                return newFilters;
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return prevFilters;
            }
        });
    };
    /*
        Sets the operator that combines a specific filter group
    */
    const setOperatorInGroup = (groupIndex, operator) => {
        setFilters((prevFilters) => {
            const newFilters = [...prevFilters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                filterGroup.operator = operator;
                return newFilters;
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return prevFilters;
            }
        });
    };
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: 'control-panel-taskpane-container default-taskpane-div' },
            React.createElement(ColumnNameCard, { columnHeader: columnHeader, columnIndex: props.selection.endingColumnIndex, mitoContainerRef: props.mitoContainerRef, gridState: props.gridState, setEditorState: props.setEditorState, mitoAPI: props.mitoAPI, setUIState: props.setUIState }),
            React.createElement("div", { className: 'default-taskpane-body-div mb-30px' },
                props.tab === ControlPanelTab.FilterSort &&
                    React.createElement(React.Fragment, null,
                        React.createElement(DtypeCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, columnFormula: columnFormula, mitoAPI: props.mitoAPI, lastStepIndex: props.lastStepIndex, lastStepType: props.lastStepType }),
                        React.createElement(SortCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI }),
                        React.createElement(FilterCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, filters: filters, columnMitoType: columnMitoType, operator: operator, mitoAPI: props.mitoAPI, setOperator: setOperator, setFilterInGroup: setFilterInGroup, setOperatorInGroup: setOperatorInGroup, deleteFilterFromGroup: deleteFilterFromGroup, addFilterToGroup: addFilterToGroup, setFilter: setFilter, deleteFilter: deleteFilter, addFilter: addFilter, addFilterGroup: addFilterGroup })),
                props.tab === ControlPanelTab.UniqueValues &&
                    React.createElement(React.Fragment, null,
                        React.createElement(ValuesTab, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI, filters: filters, columnMitoType: columnMitoType, toggleExclusiveFilters: toggleExclusiveFilters })),
                props.tab === ControlPanelTab.SummaryStats &&
                    React.createElement(React.Fragment, null,
                        React.createElement(ColumnSummaryGraph, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI }),
                        React.createElement(ColumnSummaryStatistics, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI })))),
        React.createElement(ControlPanelTaskpaneTabs, { selectedTab: props.tab, setUIState: props.setUIState, mitoAPI: props.mitoAPI })));
};
export default ControlPanelTaskpane;
//# sourceMappingURL=ControlPanelTaskpane.js.map