// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
import React, { useEffect, useState } from 'react';
import DefaultTaskpane from '../DefaultTaskpane';
// Import 
import TextButton from '../../elements/TextButton';
import Row from '../../spacing/Row';
import Col from '../../spacing/Col';
import Select from '../../elements/Select';
import DropdownItem from '../../elements/DropdownItem';
/*
    A taskpane that allows a user to download their current sheet.

    It does this by:
    1. Getting a string representation of the sheet through the api
    2. Encoding that as a file
    3. Allowing the user to download that file

    To see more about this process, read documentation here:
    https://blog.logrocket.com/programmatic-file-downloads-in-the-browser-9a5186298d5c/
*/
const DownloadTaskpane = (props) => {
    var _a;
    const [exportType, setExportType] = useState('csv');
    const [exportString, setExportString] = useState('');
    const emptySheet = props.sheetDataArray.length === 0;
    const numRows = (_a = props.sheetDataArray[props.selectedSheetIndex]) === null || _a === void 0 ? void 0 : _a.numRows;
    const loadExport = async () => {
        // Don't try and load data if the sheet is empty
        if (emptySheet) {
            return;
        }
        if (exportType === 'csv') {
            const csvString = await props.mitoAPI.getDataframeAsCSV(props.selectedSheetIndex);
            setExportString(csvString);
        }
        else if (exportType === 'excel') {
            const excelString = await props.mitoAPI.getDataframeAsExcel(props.selectedSheetIndex);
            setExportString(excelString);
        }
    };
    // Async load in the data from the mitoAPI
    useEffect(() => {
        setExportString('');
        void loadExport();
    }, [props.selectedSheetIndex, exportType]);
    const onDownload = () => {
        void props.mitoAPI.sendLogMessage('button_download_log_event', {
            sheet_index: props.selectedSheetIndex,
            df_name: props.dfName,
            export_type: exportType
        });
    };
    if (emptySheet) {
        return (React.createElement(DefaultTaskpane, { header: 'Nothing to Download', setUIState: props.setUIState, taskpaneBody: React.createElement("p", { className: 'text-body-1 text-overflow-wrap' }, "Import data before exporting it.") }));
    }
    let exportHRef = '';
    let exportName = '';
    if (exportType === 'csv') {
        exportHRef = URL.createObjectURL(new Blob([exportString], { type: 'text/csv' }));
        exportName = props.dfName + '.csv';
    }
    else if (exportType === 'excel') {
        exportHRef = URL.createObjectURL(new Blob(
        /*
            First, we convert the export string out of base 64 encoding,
            and the convert it back into bytes
        */
        [Uint8Array.from(window.atob(exportString), c => c.charCodeAt(0))], { type: 'text/csv' } // TODO: for some reason, text/csv works fine here
        ));
        exportName = props.dfName + '.xlsx';
    }
    return (React.createElement(DefaultTaskpane, { header: 'Download', setUIState: props.setUIState, taskpaneBody: React.createElement(React.Fragment, null,
            React.createElement(Row, { justify: 'space-between', align: 'center' },
                React.createElement(Col, null,
                    React.createElement("p", { className: 'text-header-3' }, "Export Type")),
                React.createElement(Col, null,
                    React.createElement(Select, { width: 'medium', value: exportType, onChange: (newExportType) => {
                            setExportType(newExportType);
                            setExportString('');
                        } },
                        React.createElement(DropdownItem, { title: 'csv' }),
                        React.createElement(DropdownItem, { title: 'excel', subtext: numRows > 1048576
                                ? `An Excel file holds at most 1,048,576 rows, but there ${numRows} rows in this dataframe. We'll export the first 1,048,576 rows, but this may take a while.`
                                : `Due to Python limitations, Excel export can be slower than CSV export.` })))),
            React.createElement(TextButton, { variant: 'dark', width: 'block', disabled: exportString === '', href: exportHRef, download: exportName, onClick: onDownload }, exportString === '' ? 'Preparing data for download' : `Download ${props.dfName}`)) }));
};
export default DownloadTaskpane;
//# sourceMappingURL=DownloadTaskpane.js.map