// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
import React, { useState } from 'react';
import DefaultTaskpane from '../DefaultTaskpane';
import { ModalEnum } from '../../modals/modals';
// Import 
import { TaskpaneType } from '../taskpanes';
import TextButton from '../../elements/TextButton';
import "../../../../css/taskpanes/Save/save-taskpane.css";
import Input from '../../elements/Input';
// A set of characters we don't allow in analysis names,
// just to avoid file name issues
// Taken from: https://www.mtu.edu/umc/services/digital/writing/characters-avoid/
const ILLEGAL_FILENAME_CHARACTERS = [
    '#',
    '%',
    '&',
    '{', '}',
    '/', '\\',
    '<', '>',
    '*',
    '?',
    '$',
    '!',
    '\'', '"',
    ':',
    '@'
];
function getAnalysisNameError(analysisName) {
    /*
        Helper function for validating named analyses. Avoids a set of illegal characters.

        Returns the empty string if there is no error.
    */
    if (analysisName.length == 0) {
        return 'Please enter a name for your analysis';
    }
    for (let i = 0; i < ILLEGAL_FILENAME_CHARACTERS.length; i++) {
        if (analysisName.indexOf(ILLEGAL_FILENAME_CHARACTERS[i]) > -1) {
            return `Invalid analysis name. Please remove the "${ILLEGAL_FILENAME_CHARACTERS[i]}"`;
        }
    }
    return '';
}
/*
    A taskpane that allows the user to save their
    current analysis - confirming if they try to
    overwrite an existing analysis
*/
const SaveTaskpane = (props) => {
    const [saveAnalysisName, setSaveAnalysisName] = useState(props.saveAnalysisName !== undefined ? props.saveAnalysisName : '');
    const [analysisNameError, setAnalysisNameError] = useState('');
    const clickSave = async () => {
        // Don't let the user submit if they have an invalid analysis name!
        if (getAnalysisNameError(saveAnalysisName) !== '') {
            setAnalysisNameError(`Please fix before saving: ${getAnalysisNameError(saveAnalysisName)}`);
            return;
        }
        if (props.savedAnalysisNames.indexOf(saveAnalysisName) > -1) {
            // If user is overwriting an analysis, we display a popup to let them know. 
            props.setUIState((prevUIState) => {
                return Object.assign(Object.assign({}, prevUIState), { currOpenModal: {
                        type: ModalEnum.SaveAnalysisOverwrite,
                        saveAnalysisName: saveAnalysisName
                    } });
            });
        }
        else {
            // Otherwise, we just log and save the analysis
            await props.mitoAPI.sendSaveAnalysisMessage(saveAnalysisName);
            props.setUIState((prevUIState) => {
                return Object.assign(Object.assign({}, prevUIState), { currOpenTaskpane: { type: TaskpaneType.NONE } });
            });
        }
    };
    const onChangeAnalysisName = (newAnalysisName) => {
        setSaveAnalysisName(newAnalysisName);
        setAnalysisNameError(getAnalysisNameError(newAnalysisName));
    };
    return (React.createElement(DefaultTaskpane, { header: 'Save Analysis', setUIState: props.setUIState, taskpaneBody: React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'mb-20px' },
                React.createElement("p", { className: 'text-color-error' }, "Saving and replaying an analysis is being deprecated. To save and replay an analysis, use the code that Mito generates.")),
            React.createElement("p", { className: 'text-header-3' }, "Name your analysis"),
            React.createElement(Input, { placeholder: 'Monthly Sales Analysis', value: saveAnalysisName, onChange: (e) => onChangeAnalysisName(e.target.value), autoFocus: true }),
            analysisNameError !== '' &&
                React.createElement("p", { className: 'text-color-error' }, analysisNameError),
            React.createElement("p", { className: 'text-header-3 mt-15px' }, "Existing analyses"),
            React.createElement("div", { className: 'save-analysis-analysis-selection-container' },
                props.savedAnalysisNames.length === 0 &&
                    React.createElement("p", null, "Save an analysis to have it appear here."),
                props.savedAnalysisNames.length > 0 &&
                    React.createElement("div", { className: 'save-analysis-analysis-selection' }, props.savedAnalysisNames.map((savedAnalysisName) => {
                        return (React.createElement("div", { className: 'save-analysis-analysis-name', key: savedAnalysisName, onClick: () => { onChangeAnalysisName(savedAnalysisName); } }, savedAnalysisName));
                    }))),
            React.createElement(TextButton, { variant: 'dark', width: 'block', onClick: clickSave, disabled: saveAnalysisName == '' || analysisNameError !== '' }, `Save${saveAnalysisName !== '' ? ' as ' + saveAnalysisName : ''}`)) }));
};
export default SaveTaskpane;
//# sourceMappingURL=SaveTaskpane.js.map