// Copyright (c) Mito
import React, { useState } from 'react';
import fscreen from 'fscreen';
// Import CSS
import "../../../css/toolbar.css";
// Import Types
import { ModalEnum } from '../modals/modals';
// Import Components 
import { TaskpaneType } from '../taskpanes/taskpanes';
import ToolbarButton from './ToolbarButton';
import { ToolbarButtonType } from './utils';
import { DFSource } from '../../types';
import { getColumnIndexesInSelections, isSelectionsOnlyColumnHeaders } from '../endo/selectionUtils';
const Toolbar = (props) => {
    /*
        Adds a new column directly to the right of the selected column, with new-column-{random id}
        as the name. The random id is a randomly generated 5 character sequence, which has a low
        probabilty of overlapping with other columns in terms of ID. If column IDs overlap, we have
        problems with columns getting deleted from maps.
    */
    const addColumn = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        const getNewColumnHeader = () => {
            let result = '';
            const characters = 'abcdefghijklmnopqrstuvwxyz0123456789';
            const charactersLength = characters.length;
            for (let i = 0; i < 4; i++) {
                result += characters.charAt(Math.floor(Math.random() * charactersLength));
            }
            return result;
        };
        const newColumnHeader = 'new-column-' + getNewColumnHeader();
        // The new column should be placed 1 position to the right of the last selected column
        const newColumnHeaderIndex = props.selections[props.selections.length - 1].endingColumnIndex + 1;
        void props.mitoAPI.sendColumnAddMessage(props.selectedSheetIndex, newColumnHeader, newColumnHeaderIndex);
    };
    /* Undoes the last step that was created */
    const undo = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups([TaskpaneType.PIVOT, TaskpaneType.IMPORT]);
        void props.mitoAPI.sendUndoMessage();
    };
    /* Redoes the last step that was undo, if it exists */
    const redo = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups([TaskpaneType.PIVOT]);
        void props.mitoAPI.sendRedoMessage();
    };
    const openClearAnalysisModal = () => {
        // Log that we opened this
        void props.mitoAPI.sendLogMessage('click_open_clear_analysis');
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        // Close all taskpanes if they are open, to make sure the state is not out of sync
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.ClearAnalysis }, currOpenTaskpane: { type: TaskpaneType.NONE } });
        });
    };
    /* Saves the current file as as an exported analysis */
    const downloadAnalysis = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.DOWNLOAD } });
        });
    };
    const [fullscreen, setFullscreen] = useState(false);
    fscreen.onfullscreenchange = () => {
        setFullscreen(!!fscreen.fullscreenElement);
        void props.mitoAPI.sendLogMessage('button_toggle_fullscreen', {
            // Note that this is true when _end_ in fullscreen mode, and 
            // false when we _end_ not in fullscreen mode, which is much
            // more natural than the alternative
            fullscreen: !!fscreen.fullscreenElement
        });
    };
    /*
        Toggles if Mito is full screen or not
    */
    const toggleFullscreen = () => {
        // We toggle to the opposite of whatever the fullscreen actually is (as detected by the
        // fscreen library), and then we set the fullscreen state variable to that state (in the callback
        // above), so that the component rerenders propery
        const isNotFullscreen = fscreen.fullscreenElement === undefined || fscreen.fullscreenElement === null;
        if (isNotFullscreen && props.mitoContainerRef) {
            fscreen.requestFullscreen(props.mitoContainerRef);
        }
        else {
            fscreen.exitFullscreen();
        }
    };
    const openDocumentation = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // We log the opening of the documentation taskpane
        void props.mitoAPI.sendLogMessage('clicked_documentation', {
            stage: 'opened'
        });
        // Open the documentation in a new tab - to importing because they have mito
        // installed already
        window.open('https://docs.trymito.io/how-to/importing-data-to-mito', '_blank');
    };
    const openMerge = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // We open the merge taskpane
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.MERGE } });
        });
    };
    const openDropDuplicates = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // We open the merge taskpane
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.DROP_DUPLICATES } });
        });
    };
    const openPivotTable = async () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // We check if the currently opened sheet is a result of a pivot table
        // and if so then we open the existing pivot table here, rather than
        // create a new pivot table. That is: if a user is on a pivot table, then
        // we let them edit that pivot table
        if (props.dfSources[props.selectedSheetIndex] === DFSource.Pivoted) {
            const existingPivotParams = await props.mitoAPI.getPivotParams(props.selectedSheetIndex);
            if (existingPivotParams !== undefined) {
                props.setUIState(prevUIState => {
                    return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: {
                            type: TaskpaneType.PIVOT,
                            destinationSheetIndex: props.selectedSheetIndex,
                            existingPivotParams: existingPivotParams
                        } });
                });
                return;
            }
        }
        /*
            This case just opens a new pivot table.
            
            BUG: when the user has a pivot table, and deletes a dataframe
            before it, we enter this case, as the df source of the pivot
            table is Pivoted, but the sheet index used to get the pivot
            params is out of date.

            In this case, we don't edit the pivot table, but rather open a
            new one.

            The fix for this bug is moving from sheet index -> sheet id, as
            we did with column header ids!
        */
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: {
                    type: TaskpaneType.PIVOT,
                    destinationSheetIndex: undefined,
                    existingPivotParams: undefined
                } });
        });
    };
    const deleteColumn = async () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        if (isSelectionsOnlyColumnHeaders(props.selections)) {
            const columnIndexesSelected = getColumnIndexesInSelections(props.selections);
            const columnIDsToDelete = columnIndexesSelected.map(colIdx => { var _a, _b; return ((_b = (_a = props.sheetData) === null || _a === void 0 ? void 0 : _a.data[colIdx]) === null || _b === void 0 ? void 0 : _b.columnID) || ''; }).filter(columnID => columnID !== '');
            if (columnIDsToDelete !== undefined) {
                await props.mitoAPI.sendDeleteColumn(props.selectedSheetIndex, columnIDsToDelete);
            }
        }
    };
    const openSave = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.SAVE } });
        });
    };
    const openReplay = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.REPLAY_ANALYSIS } });
        });
    };
    const openImport = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.IMPORT } });
        });
    };
    const openGraph = () => {
        // We turn off editing mode, if it is on
        props.setEditorState(undefined);
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.GRAPH, graphSidebarSheet: props.selectedSheetIndex } });
        });
    };
    const openSteps = () => {
        void props.mitoAPI.sendLogMessage('click_open_steps');
        props.setUIState(prevUIState => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenModal: { type: ModalEnum.None }, currOpenTaskpane: { type: TaskpaneType.STEPS } });
        });
    };
    const catchup = () => {
        // Fast forwards to the most recent step, allowing editing
        void props.mitoAPI.sendLogMessage('click_catch_up');
        void props.mitoAPI.checkoutStepByIndex(props.lastStepIndex);
    };
    return (React.createElement("div", { className: 'toolbar-container' },
        React.createElement("div", { className: 'toolbar-left-half' },
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.UNDO, buttonTitle: "UNDO", buttonSubtext: 'Undo the most recent edit', onClick: undo }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.REDO, buttonTitle: "REDO", buttonSubtext: 'Redo the most recently undone edit', onClick: redo }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.CLEAR, buttonTitle: "CLEAR", buttonSubtext: 'Clear all edits made to any imported data', onClick: openClearAnalysisModal }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.IMPORT, buttonTitle: "IMPORT", buttonSubtext: 'Import data from a file', onClick: openImport }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.EXPORT, buttonTitle: "EXPORT", buttonSubtext: 'Export the current dataframe to a CSV', onClick: downloadAnalysis }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.ADD_COL, buttonTitle: "ADD COL", buttonSubtext: 'Add a new column to this dataframe', onClick: addColumn, highlightToolbarButton: props.highlightAddColButton }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.DEL_COL, buttonTitle: "DEL COL", buttonSubtext: 'Delete the currently selected column from this dataframe', onClick: deleteColumn }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.PIVOT, buttonTitle: "PIVOT", buttonSubtext: 'Generate a pivot table', onClick: openPivotTable, highlightToolbarButton: props.highlightPivotTableButton }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.MERGE, buttonTitle: "MERGE", buttonSubtext: 'Merge two dataframes into a new dataframe', onClick: openMerge }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.DROP_DUPLICATES, buttonTitle: "DEDUPE", buttonSubtext: 'Drop duplicated rows in a dataframe', onClick: openDropDuplicates }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.GRAPH, buttonTitle: "GRAPH", buttonSubtext: 'Generate interactive graphs', onClick: openGraph }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.SAVE, buttonTitle: "SAVE", buttonSubtext: 'Save the current analysis so these edits can be replayed in the future', onClick: openSave }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.REPLAY, buttonTitle: "REPLAY", buttonSubtext: 'Replay an analysis that has been previously saved', onClick: openReplay }),
            React.createElement("div", { className: "toolbar-vertical-line" })),
        React.createElement("div", { className: 'toolbar-right-half' },
            props.currStepIdx !== props.lastStepIndex &&
                React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.CATCH_UP, buttonTitle: "CATCH UP", buttonSubtext: 'Stop looking at a previous step and catch up to the most recent edit', onClick: catchup }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.STEPS, buttonTitle: "STEPS", buttonSubtext: 'See a list of edits performed in this analysis', onClick: openSteps }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.DOCS, buttonTitle: "DOCS", buttonSubtext: 'Open a documentation taskpane with the sheet functions', onClick: openDocumentation }),
            React.createElement(ToolbarButton, { toolbarButtonType: fullscreen ? ToolbarButtonType.CLOSE_FULLSCREEN : ToolbarButtonType.OPEN_FULLSCREEN, buttonTitle: "FULLSCREEN", buttonSubtext: 'Enter full screen mode', onClick: toggleFullscreen }))));
};
export default Toolbar;
//# sourceMappingURL=Toolbar.js.map